"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ApiConsoleApp = void 0;
const body_parser_1 = require("body-parser");
const command_line_args_1 = __importDefault(require("command-line-args"));
const cors_1 = __importDefault(require("cors"));
const express_1 = __importDefault(require("express"));
const swagger_ui_express_1 = require("swagger-ui-express");
const ts_lambda_api_1 = require("ts-lambda-api");
/**
 * Simple console application that hosts an express HTTP
 * server.
 *
 * A request is mapped from the HTTP request to the ApiServer
 * event format, a ApiServer instance then processes the request.
 * The ApiServer response is mapped to a standard HTTP response and
 * returned to the express client.
 */
class ApiConsoleApp extends ts_lambda_api_1.ApiApp {
    /**
     * Builds an new console app.
     *
     * If a value for`appConfig` is not passed, the `serverLogger`
     * property is missing or the `serverLogger.logTimestamp` property
     * is missing timestamps will be enabled for logger output.
     */
    constructor(controllersPath, appConfig, appContainer) {
        if (!appConfig) {
            appConfig = new ts_lambda_api_1.AppConfig();
        }
        if (!appConfig.serverLogger) {
            appConfig.serverLogger = {
                format: "string",
                level: ts_lambda_api_1.LogLevel.info,
                logTimestamp: true
            };
        }
        if (appConfig.serverLogger.logTimestamp === null ||
            appConfig.serverLogger.logTimestamp === undefined) {
            appConfig.serverLogger.logTimestamp = true;
        }
        super(controllersPath, appConfig, appContainer);
        this.logger = this.logFactory.getLogger(ApiConsoleApp);
        this.expressApp = express_1.default();
    }
    async run(request, context) {
        return await this.apiServer.processEvent(request, context);
    }
    /**
     * Starts the express server.
     *
     * @param args Command line arguments for this server, see --help for more info.
     */
    async runServer(args) {
        let self = this;
        let options = this.parseArguments(args);
        let listenOnAllHosts = options.host === "*";
        let baseUrl = `http://${options.host}:${options.port}`;
        this.logger.debug("Server base URL: %s", baseUrl);
        await super.initialiseControllers();
        this.configureServer(baseUrl, options);
        this.expressApp.all("*", (req, res, next) => self.handleHttpRequest(self, req, res, next));
        this.server = await new Promise((resolve, reject) => {
            try {
                if (listenOnAllHosts) {
                    this.logger.debug("Listening on all hosts");
                    let server = this.expressApp.listen(options.port, () => resolve(server));
                }
                else {
                    this.logger.debug("Listening on host: %s", options.host);
                    let server = this.expressApp.listen(options.port, options.host, () => resolve(server));
                }
            }
            catch (ex) {
                reject(ex);
            }
        });
        this.logger.info(`Listening for HTTP requests on ${baseUrl} ...`);
    }
    configureServer(baseUrl, options) {
        // setup CORS
        this.expressApp.use(cors_1.default({
            origin: options["cors-origin"]
        }));
        this.logger.debug("CORS origin set to: %s", options["cors-origin"]);
        // setup body parser to Base64 encode request's
        this.expressApp.use(body_parser_1.raw({
            limit: ApiConsoleApp.MAX_REQUEST_BODY_SIZE,
            type: r => true
        }));
        this.logger.warn("To simulate an AWS ALB or Application Gateway the max size for requests is limited to %s", ApiConsoleApp.MAX_REQUEST_BODY_SIZE);
        // setup SwaggerUI
        if (this.appConfig.openApi.enabled) {
            this.configureSwaggerUi(baseUrl);
        }
    }
    configureSwaggerUi(baseUrl) {
        let basePath = this.appConfig.base || "";
        let specUrl = `${basePath}/open-api.json`;
        let swaggerUiUrl = `${basePath}/swagger`;
        this.logger.info("OpenAPI enabled, configuring SwaggerUI to be available @ %s%s", baseUrl, swaggerUiUrl);
        this.expressApp.use(swaggerUiUrl, swagger_ui_express_1.serve, swagger_ui_express_1.setup(null, {
            explorer: true,
            swaggerUrl: specUrl
        }));
    }
    async stopServer() {
        if (!this.server) {
            throw new Error("stopServer can only be called after runServer has been called and has completed");
        }
        this.logger.info("Server shutting down");
        await new Promise((resolve, reject) => {
            try {
                this.server.close(() => resolve());
            }
            catch (ex) {
                reject(ex);
            }
        });
    }
    parseArguments(args) {
        this.logger.trace("Command line arguments: %s", args);
        return command_line_args_1.default(ApiConsoleApp.APP_OPTIONS, {
            argv: args
        });
    }
    async handleHttpRequest(self, request, response, onError) {
        try {
            this.logger.debug("Mapping express request to AWS model");
            let apiRequestEvent = await self.mapRequestToApiEvent(request);
            let apiResponse = await self.run(apiRequestEvent, {});
            self.forwardApiResponse(apiResponse, response);
        }
        catch (ex) {
            onError(ex);
        }
    }
    async mapRequestToApiEvent(request) {
        let apiRequest = new ts_lambda_api_1.ApiRequest();
        apiRequest.httpMethod = request.method;
        apiRequest.path = request.path;
        Object.keys(request.headers)
            .forEach(h => apiRequest.headers[h] = request.headers[h]);
        Object.keys(request.query)
            .forEach(q => apiRequest.queryStringParameters[q] = request.query[q]);
        if (!ApiConsoleApp.HTTP_METHODS_WITH_ENTITY.includes(request.method)) {
            // request HTTP method does not include a body
            return apiRequest;
        }
        // read request body as a base64 string
        this.logger.debug("Reading request body as base64 string");
        let body = request.body;
        apiRequest.body = body.toString("base64");
        apiRequest.isBase64Encoded = true;
        return apiRequest;
    }
    forwardApiResponse(apiResponse, response) {
        this.logger.debug("Mapping AWS response model to express response");
        let headers = apiResponse.headers;
        response.status(apiResponse.statusCode);
        Object.keys(headers).forEach(h => response.header(h, headers[h]));
        if (apiResponse.isBase64Encoded) {
            response.contentType(apiResponse.headers["content-type"] || "application/octet-stream");
            response.end(Buffer.from(apiResponse.body, "base64"));
        }
        else {
            response.send(apiResponse.body);
        }
    }
}
ApiConsoleApp.MAX_REQUEST_BODY_SIZE = "6144kb";
ApiConsoleApp.HTTP_METHODS_WITH_ENTITY = ["POST", "PUT", "PATCH"];
ApiConsoleApp.APP_OPTIONS = [
    { name: "port", alias: "p", type: Number, defaultValue: 8080 },
    { name: "host", alias: "h", type: String, defaultValue: "*" },
    { name: "cors-origin", alias: "c", type: String, defaultValue: "*" }
];
__decorate([
    ts_lambda_api_1.timed,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [ts_lambda_api_1.ApiRequest, Object]),
    __metadata("design:returntype", Promise)
], ApiConsoleApp.prototype, "run", null);
__decorate([
    ts_lambda_api_1.timed,
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [ApiConsoleApp, Object, Object, Function]),
    __metadata("design:returntype", Promise)
], ApiConsoleApp.prototype, "handleHttpRequest", null);
exports.ApiConsoleApp = ApiConsoleApp;
//# sourceMappingURL=ApiConsoleApp.js.map